#!/bin/bash
# as of 2007/07/11 15:21:00
#
#                            :-)  ./g_puckering  (-:
#
#DESCRIPTION
#-----------
#this is a small test program meant to serve as a template when writing your
#own analysis tools. The advantage of using gromacs for this is that you have
#access to all information in the topology, and your program will be able to
#handle all types of coordinates and trajectory files supported by gromacs. Go
#ahead and try it! This test version just writes the coordinates of an
#arbitrary atom to standard out for each frame. You can select which atom you
#want to examine with the -n argument.
#
#Option     Filename  Type         Description
#------------------------------------------------------------
#  -s      topol.tpr  Input        Structure+mass(db): tpr tpb tpa gro g96 pdb
#                                   xml
#  -f       traj.xtc  Input        Generic trajectory: xtc trr trj gro g96 pdb
#  -n      index.ndx  Input        Index file
# -ot   cp_theta.xvg  Output, Opt. xvgr/xmgr file
# -op     cp_phi.xvg  Output, Opt. xvgr/xmgr file
# -oq       cp_q.xvg  Output, Opt. xvgr/xmgr file
#-otp      cp_tp.xvg  Output, Opt. xvgr/xmgr file
#-opol  cp_polar.xvg  Output, Opt. xvgr/xmgr file
#-odt  cp_dtheta.xvg  Output, Opt. xvgr/xmgr file
#-odp    cp_dphi.xvg  Output, Opt. xvgr/xmgr file
#
#      Option   Type  Value  Description
#------------------------------------------------------
#      -[no]h   bool    yes  Print help info and quit
#      -[no]X   bool     no  Use dialog box GUI to edit command line options
#       -nice    int      0  Set the nicelevel
#          -b   time      0  First frame (ps) to read from trajectory
#          -e   time      0  Last frame (ps) to read from trajectory
#         -dt   time      0  Only use frame when t MOD dt = first time (ps)
#      -[no]w   bool     no  View output xvg, xpm, eps and pdb files
#   -[no]xvgr   bool    yes  Add specific codes (legends etc.) in the output
#                            xvg files for the xmgrace program
#          -i    int      6  Size of Ring
#      -[no]v   bool     no  Be loud and noisy
shopt -s extglob
_g_puckering_compl() {
local p c
COMPREPLY=() c=${COMP_WORDS[COMP_CWORD]} p=${COMP_WORDS[COMP_CWORD-1]}
if (( $COMP_CWORD <= 1 )) || [[ $c == -* ]]; 
then COMPREPLY=( $(compgen  -W ' -f -s -n -ot -op -oq -otp -opol -odt -odp -h -nice -b -e -w -noxvgr -dt' -- $c)); return 0; fi
case "$p" in
-s) COMPREPLY=( $(compgen -X '!*.+(tpr|tpb|tpa)*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-f) COMPREPLY=( $(compgen -X '!*.+(xtc|trr|trj|gro|g96|pdb|g87)*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-n) COMPREPLY=( $(compgen -X '!*.ndx*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-ot)   COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-op)   COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-oq)   COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-otp)  COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-opol) COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-odt)  COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
-odp)  COMPREPLY=( $(compgen -X '!*.xvg*(.gz|.Z)' -f $c ; compgen -S '/' -X '.*' -d $c ));;
#-type) COMPREPLY=( $(compgen -W ' angle dihedral improper ryckaert-bellemans ' -- $c ));;
esac }
complete -F _g_puckering_compl g_puckering
