/*
 * $Id$
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * Written by Oliver Stueker
 * 
 * Acknowledgements:
 *  The implementation of Cremer-Pople-Puckering Algorithm
 *  was done by Mark J. Forster for his GPL-program mdxvu
 *  http://sourceforge.net/projects/mdxvu/
 *
 * 
 * And Hey:
 * Giving Russians Opium May Alter Current Situation
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include <string.h>
#include <stdio.h>


float x[25],y[25],z[25];

void cremer_pople(float *coords_x, float *coords_y, float *coords_z, int nring,
                  float *f1,float *f2,float *f3);

int gmx_puckering(int argc, char *argv[]);

int main(int argc, char *argv[])
{
    gmx_puckering(argc,argv);
    return 0;
}

int gmx_puckering(int argc, char *argv[])
{
    int i;
    int atnum[7];
    float coords_x[7], coords_y[7], coords_z[7];
    float cptheta, cpphi, cpQ;

    // T-o-D-o  : Read Index File
    // Testcase: Glycose 
    atnum[1]=6; 
    atnum[2]=14;
    atnum[3]=13;
    atnum[4]=12;
    atnum[5]=8;
    atnum[6]=5;

    // T-o-D-o: Read Trajectory
    // Testcase: Glycose 
    x[1]=-3.051;  y[1]=0.403;   z[1]=-1.678;     // H61 
    x[2]=-2.010;  y[2]=0.492;   z[2]=-1.990;     // C6  
    x[3]=-1.674;  y[3]=-0.592;  z[3]=-2.830;     // O6  
    x[4]=-0.869;  y[4]=-0.423;  z[4]=-3.409;     // H60 
    x[5]=-1.125;  y[5]=0.521;   z[5]=-0.701;     // C5  
    x[6]=-1.323;  y[6]=-0.617;  z[6]=0.156;      // O5  
    x[7]=-1.551;  y[7]=1.356;   z[7]=-0.210;     // H5  
    x[8]=0.395;   y[8]=0.693;   z[8]=-1.070;     // C4  
    x[9]=0.629;   y[9]=-0.205;  z[9]=-1.585;     // H4  
    x[10]=0.580;  y[10]=1.863;  z[10]=-1.844;    // O4  
    x[11]=1.002;  y[11]=2.430;  z[11]=-1.113;    // H40 
    x[12]=1.301;  y[12]=0.688;  z[12]=0.186;     // C3  
    x[13]=1.076;  y[13]=-0.531; z[13]=1.004;     // C2  
    x[14]=-0.509; y[14]=-0.675; z[14]=1.337;     // C1  
    x[15]=-0.796; y[15]=-1.829; z[15]=2.016;     // O1  
    x[16]=-0.223; y[16]=-2.481; z[16]=1.660;     // H10 
    x[17]=-0.764; y[17]=0.197;  z[17]=1.952;     // H1  
    x[18]=1.820;  y[18]=-0.492; z[18]=2.245;     // O2  
    x[19]=1.891;  y[19]=0.435;  z[19]=2.532;     // H20 
    x[20]=1.518;  y[20]=-1.339; z[20]=0.496;     // H2  
    x[21]=0.982;  y[21]=1.552;  z[21]=0.773;     // H3  
    x[22]=2.673;  y[22]=0.848;  z[22]=-0.175;    // O3  
    x[23]=3.110;  y[23]=1.199;  z[23]=0.642;     // H30 
    x[24]=-1.848; y[24]=1.491;  z[24]=-2.517;    // H62 

    for(i=1 ; i<=6 ; i++ )
    {
        coords_x[i]=x[atnum[i]];
        coords_y[i]=y[atnum[i]];
        coords_z[i]=z[atnum[i]];
    }

    cremer_pople(coords_x, coords_y, coords_z, 6, &cptheta, &cpphi, &cpQ);
    printf("CREMER POPLE theta  = %8.2f \n",cptheta);
    printf("CREMER POPLE phi2   = %8.2f \n",cpphi);
    printf("CREMER POPLE Q      = %8.2f \n",cpQ);
    
    return 0;


}


/*------------------------------------------------------------
function cremer_pople
        compute Cremer-Pople ring puckering parameters
        for 5 and 6 membered rings, return results
        in float f1,f2,f3.

        This functions original code was part of mdxvu, 
        written by Mark J Forster, in September 2005 
        http://sourceforge.net/projects/mdxvu/
------------------------------------------------------------*/
void cremer_pople(float *coords_x, float *coords_y, float *coords_z, int nring,
                  float *f1,float *f2,float *f3)
    /** 
     *  coords_x, 
     *  coords_y,
     *  coords_z    are a matrix containg all cartesian coordinates of relevant ring atoms
     *  nring       determens if it's a five- or six-membered ring
     *
     *  f1,f2,f3 are pointers to floats
     *  used to carry out results theta,phi2,Q 
     */
    {
    int i,j,k;
    float xnew[7],ynew[7],znew[7];
    float zdisp[7];
    float xsum,ysum,zsum;
    float xmean,ymean,zmean;
    float R1[3],R2[3],n[3];
    float length,cross[3];
    float pi=3.14159;
    float s,c,NN;
    float Q,sum1,sum2,q2,q3,phi2;
    float sq,theta,term1,term2;
    float stheta;

    /* for hexapyranose ring should pick
       atoms O5,C1,C2,C3,C4,C5
       for agreement with Cremer & Pople Sucrose
       pyranoid ring coordinates
    */
    /* for furanoid ring pick atoms
       O2', C5', C4', C3' C2'
    */
    NN=((float) nring);
    /* copy coords to arrays xnew,ynew,znew */
    for (i=1;i<=nring;i++) {
            xnew[i]=coords_x[i];
            ynew[i]=coords_y[i];
            znew[i]=coords_z[i];
            };

    /* find mean x,y,z coordinates */
    zsum=0.0;
    ysum=0.0;
    xsum=0.0;

    for (i=1;i<=nring;i++) {
            xsum += xnew[i];
            ysum += ynew[i];
            zsum += znew[i];
            };

    xmean=xsum/NN;
    ymean=ysum/NN;
    zmean=zsum/NN;

    /* now subtract mean -> origin at geometric
       mean of coordinates of 6 ring atoms */
    for (i=1;i<=nring;i++) {
            xnew[i] -= xmean;
            ynew[i] -= ymean;
            znew[i] -= zmean;
            };


    /* compute R1,R2 vectors */
    R1[0]=0.0;
    R1[1]=0.0;
    R1[2]=0.0;
    R2[0]=0.0;
    R2[1]=0.0;
    R2[2]=0.0;
    for (i=1;i<=nring;i++) {
            s=sin(2*pi*(i-1)/NN);
            c=cos(2*pi*(i-1)/NN);
            R1[0] += xnew[i]*s;
            R1[1] += ynew[i]*s;
            R1[2] += znew[i]*s;
            R2[0] += xnew[i]*c;
            R2[1] += ynew[i]*c;
            R2[2] += znew[i]*c;
            };
    /*
    printf("vector R1 = %f %f %f \n",R1[0],R1[1],R1[2]);
    printf("vector R2 = %f %f %f \n",R2[0],R2[1],R2[2]);
    */

    /* compute cross product */
    cross[0] = R1[1]*R2[2]-R1[2]*R2[1];
    cross[1] = R1[2]*R2[0]-R1[0]*R2[2];
    cross[2] = R1[0]*R2[1]-R1[1]*R2[0];
    /* compute length */
    length=sqrt(cross[0]*cross[0]+cross[1]*cross[1]+cross[2]*cross[2]);
    /* now compute n vector */
    n[0] = cross[0]/length;
    n[1] = cross[1]/length;
    n[2] = cross[2]/length;
    /*
    printf("n vector = %f %f %f \n",n[0],n[1],n[2]);
    */

    /* compute z axis displacements */
    /* EQ 11 OF CREMER AND POPLE PAPER */
    for (i=1;i<=nring;i++) {
    //      k=atnum[i];
            zdisp[i]=xnew[i]*n[0]+ynew[i]*n[1]+znew[i]*n[2];
    /*      printf("atom %d %4.4s z coord = %f z disp = %f \n",
                    i,atom_name[k],znew[i],zdisp[i]); */
            };

    /* compute total puckering amplitude */
    sum1=0.0;
    for (i=1;i<=nring;i++) sum1 += (zdisp[i]*zdisp[i]);
    Q=sqrt(sum1);
    /* printf("total puckering amplitude = %5.4f \n",Q); */

    /* now compute q1, phi1 using Eq 12,13 of
       D.Cremer and J.A.Pople,
       JACS vol 97, 1354 (1975) */
    /* q1 should be = 0 hence no need to evaluate */

    if (nring==5 || nring==6) {
    /* now compute q2, phi2 using Eq 12,13 of
       D.Cremer and J.A.Pople,
       JACS vol 97, 1354 (1975) */
    sum1=0.0;
    sum2=0.0;
    sq=sqrt(2.0/NN);
    for (j=1;j<=nring;j++) {
            term1=(zdisp[j]*cos(4*pi*(j-1)/NN));
            term2=(zdisp[j]*sin(4*pi*(j-1)/NN));
            sum1=sum1+term1;
            sum2=sum2-term2;
            };
    sum1 *= sq;
    sum2 *= sq;
    /* printf("sum1,sum2= %f %f \n",sum1,sum2); */
    q2=sqrt(sum1*sum1+sum2*sum2);
    /* printf("q2 = %8.4f \n",q2); */
    /* acos returns value in range 0-180 degrees */
    phi2=acos((double) (sum1/q2));
    /* convert to degrees */
    phi2=phi2*180.0/pi;
    stheta=sum2/q2;
    if (stheta<0.0) phi2=(360.0-phi2);
    /* printf("phi 2 = %8.4f degrees \n",phi2*180/pi); */
    }; /* end if nring */

    if (nring==6) {
    /* compute q3 using eq 14 of Cremer & Pople */
    q3=0.0;
    for (j=1;j<=nring;j++) {
            if ((j % 2)==0) {
                    q3 -= zdisp[j];
            } else {
                    q3 += zdisp[j];
            }; /* end if */
    }; /* next j */
    q3=q3/sqrt(NN);
    /* printf("q3 = %8.4f \n",q3); */
    theta=acos(q3/Q);
    /* convert to degrees */
    theta=theta*180.0/pi;
    /* printf("theta = %8.4f degrees \n",theta); */
    }; /* end if nring==6 */
    /* take angle results back to main program */
    *f1=theta;
    *f2=phi2;
    *f3=Q;

}
