/*
 * $Id: gmx_puckering.h 22 2009-06-08 15:21:24Z revilo $
 *
 *                This source code is part of
 *
 *                        g_puckering
 *
 * g_puckering is an analysis-tool for calculating the Cremer-Pople
 * ring puckering parameters within a given GROMACS trajectory file.
 *
 * written 2007 by Oliver Stueker
 *
 * Acknowledgments:
 *  The Analysis tool bases on a template and several functions from:
 *  GROMACS - GROningen MAchine for Chemical Simulations
 *  an open source program to perform molecular dynamics simulations
 *  under GNU General Public License.
 *  Written by David van der Spoel, Erik Lindahl, Berk Hess, and others.
 *  check out http://www.gromacs.org for more information
 *
 *  The implementation of Cremer-Pople-Puckering Algorithm
 *  was done by Mark J. Forster for his GPL-program mdxvu
 *  http://sourceforge.net/projects/mdxvu/
 *
 * License:
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

 /* TYPES */
 enum plotType{
   ePlotTHETA, ePlotPHI, ePlotQ, ePlotRAMA, ePlotPOLAR
 };

 struct SRing {
   float         aXcoords[6];
   float         aYcoords[6];
   float         aZcoords[6];
   float         fTheta;
   float         fPhi;
   float         fQ;
   struct SRing *next;
 };

 struct SFrame {
   int           nNumber;
   float         fTime;
   struct SRing *rings;
   struct SFrame *next;
 };

 /* PROTOTYPES */
  /* cremer pople.c */
  void cremer_pople(float *coords_x, float *coords_y, float *coords_z, int nring,
                    float *f1,float *f2,float *f3);
    /**
     *  coords_x,
     *  coords_y,
     *  coords_z    are a matrix containg all cartesian coordinates of relevant ring atoms
     *  nring       determens if it's a five- or six-membered ring
     *
     *  f1,f2,f3 are pointers to floats
     *  used to carry out results theta,phi2,Q
     */

  /* gmx_puckering.c */
  void writeTimePlot(char *fn, int type, char *grpnm, struct SFrame *firstFrame);
    /**
     * fn       filename
     * type     plotType
     * grpnm    Name of Indexgroup
     * firstFrame pointer to Linked List
     */
  void writePolarPlot(char *fn, char *grpnm, struct SFrame *firstFrame ) ;
    /**
     * fn       filename
     * grpnm    Name of Indexgroup
     * firstFrame pointer to Linked List
     */

  void writeCpDistributionPlot (char *fn, int type, char *grpnm, struct SFrame *firstFrame ) ;
    /**
     * fn       filename
     * type     plotType
     * grpnm    Name of Indexgroup
     * firstFrame pointer to Linked List
     */

  void selfTest();

  void dumpFrames(struct SFrame *firstFrame);


 /* DEFINES */

 /* TYPEDEFS */

/* END */

